diff src/procimpl.h @ 853:2b6fa2618f76

[xemacs-hg @ 2002-05-28 08:44:22 by ben] merge my stderr-proc ws make-docfile.c: Fix places where we forget to check for EOF. code-init.el: Don't use CRLF conversion by default on process output. CMD.EXE and friends work both ways but Cygwin programs don't like the CRs. code-process.el, multicast.el, process.el: Removed. Improvements to call-process-internal: -- allows a buffer to be specified for input and stderr output -- use it on all systems -- implement C-g as documented -- clean up and comment call-process-region uses new call-process facilities; no temp file. remove duplicate funs in process.el. comment exactly how coding systems work and fix various problems. open-multicast-group now does similar coding-system frobbing to open-network-stream. dumped-lisp.el, faces.el, msw-faces.el: Fix some hidden errors due to code not being defined at the right time. xemacs.mak: Add -DSTRICT. ================================================================ ALLOW SEPARATION OF STDOUT AND STDERR IN PROCESSES ================================================================ Standard output and standard error can be processed separately in a process. Each can have its own buffer, its own mark in that buffer, and its filter function. You can specify a separate buffer for stderr in `start-process' to get things started, or use the new primitives: set-process-stderr-buffer process-stderr-buffer process-stderr-mark set-process-stderr-filter process-stderr-filter Also, process-send-region takes a 4th optional arg, a buffer. Currently always uses a pipe() under Unix to read the error output. (#### Would a PTY be better?) sysdep.h, sysproc.h, unexfreebsd.c, unexsunos4.c, nt.c, emacs.c, callproc.c, symsinit.h, sysdep.c, Makefile.in.in, process-unix.c: Delete callproc.c. Move child_setup() to process-unix.c. wait_for_termination() now only needed on a few really old systems. console-msw.h, event-Xt.c, event-msw.c, event-stream.c, event-tty.c, event-unixoid.c, events.h, process-nt.c, process-unix.c, process.c, process.h, procimpl.h: Rewrite the process methods to handle a separate channel for error input. Create Lstreams for reading in the error channel. Many process methods need change. In general the changes are fairly clear as they involve duplicating what's used for reading the normal stdout and changing for stderr -- although tedious, as such changes are required throughout the entire process code. Rewrote the code that reads process output to do two loops, one for stdout and one for stderr. gpmevent.c, tooltalk.c: set_process_filter takes an argument for stderr. ================================================================ NEW ERROR-TRAPPING MECHANISM ================================================================ Totally rewrite error trapping code to be unified and support more features. Basic function is call_trapping_problems(), which lets you specify, by means of flags, what sorts of problems you want trapped. these can include -- quit -- errors -- throws past the function -- creation of "display objects" (e.g. buffers) -- deletion of already-existing "display objects" (e.g. buffers) -- modification of already-existing buffers -- entering the debugger -- gc -- errors->warnings (ala suspended errors) etc. All other error funs rewritten in terms of this one. Various older mechanisms removed or rewritten. window.c, insdel.c, console.c, buffer.c, device.c, frame.c: When creating a display object, added call to note_object_created(), for use with trapping_problems mechanism. When deleting, call check_allowed_operation() and note_object deleted(). The trapping-problems code records the objects created since the call-trapping-problems began. Those objects can be deleted, but none others (i.e. previously existing ones). bytecode.c, cmdloop.c: internal_catch takes another arg. eval.c: Add long comments describing the "five lists" used to maintain state (backtrace, gcpro, specbind, etc.) in the Lisp engine. backtrace.h, eval.c: Implement trapping-problems mechanism, eliminate old mechanisms or redo in terms of new one. frame.c, gutter.c: Flush out the concept of "critical display section", defined by the in_display() var. Use an internal_bind() to get it reset, rather than just doing it at end, because there may be a non-local exit. event-msw.c, event-stream.c, console-msw.h, device.c, dialog-msw.c, frame.c, frame.h, intl.c, toolbar.c, menubar-msw.c, redisplay.c, alloc.c, menubar-x.c: Make use of new trapping-errors stuff and rewrite code based on old mechanisms. glyphs-widget.c, redisplay.h: Protect calling Lisp in redisplay. insdel.c: Protect hooks against deleting existing buffers. frame-msw.c: Use EQ, not EQUAL in hash tables whose keys are just numbers. Otherwise we run into stickiness in redisplay because internal_equal() can QUIT. ================================================================ SIGNAL, C-G CHANGES ================================================================ Here we change the way that C-g interacts with event reading. The idea is that a C-g occurring while we're reading a user event should be read as C-g, but elsewhere should be a QUIT. The former code did all sorts of bizarreness -- requiring that no QUIT occurs anywhere in event-reading code (impossible to enforce given the stuff called or Lisp code invoked), and having some weird system involving enqueue/dequeue of a C-g and interaction with Vquit_flag -- and it didn't work. Now, we simply enclose all code where we want C-g read as an event with {begin/end}_dont_check_for_quit(). This completely turns off the mechanism that checks (and may remove or alter) C-g in the read-ahead queues, so we just get the C-g normal. Signal.c documents this very carefully. cmdloop.c: Correct use of dont_check_for_quit to new scheme, remove old out-of-date comments. event-stream.c: Fix C-g handling to actually work. device-x.c: Disable quit checking when err out. signal.c: Cleanup. Add large descriptive comment. process-unix.c, process-nt.c, sysdep.c: Use QUIT instead of REALLY_QUIT. It's not necessary to use REALLY_QUIT and just confuses the issue. lisp.h: Comment quit handlers. ================================================================ CONS CHANGES ================================================================ free_cons() now takes a Lisp_Object not the result of XCONS(). car and cdr have been renamed so that they don't get used directly; go through XCAR(), XCDR() instead. alloc.c, dired.c, editfns.c, emodules.c, fns.c, glyphs-msw.c, glyphs-x.c, glyphs.c, keymap.c, minibuf.c, search.c, eval.c, lread.c, lisp.h: Correct free_cons calling convention: now takes Lisp_Object, not Lisp_Cons chartab.c: Eliminate direct use of ->car, ->cdr, should be black box. callint.c: Rewrote using EXTERNAL_LIST_LOOP to avoid use of Lisp_Cons. ================================================================ USE INTERNAL-BIND-* ================================================================ eval.c: Cleanups of these funs. alloc.c, fileio.c, undo.c, specifier.c, text.c, profile.c, lread.c, redisplay.c, menubar-x.c, macros.c: Rewrote to use internal_bind_int() and internal_bind_lisp_object() in place of whatever varied and cumbersome mechanisms were formerly there. ================================================================ SPECBIND SANITY ================================================================ backtrace.h: - Improved comments backtrace.h, bytecode.c, eval.c: Add new mechanism check_specbind_stack_sanity() for sanity checking code each time the catchlist or specbind stack change. Removed older prototype of same mechanism. ================================================================ MISC ================================================================ lisp.h, insdel.c, window.c, device.c, console.c, buffer.c: Fleshed out authorship. device-msw.c: Correct bad Unicode-ization. print.c: Be more careful when not initialized or in fatal error handling. search.c: Eliminate running_asynch_code, an FSF holdover. alloc.c: Added comments about gc-cons-threshold. dialog-x.c: Use begin_gc_forbidden() around code to build up a widget value tree, like in menubar-x.c. gui.c: Use Qunbound not Qnil as the default for gethash. lisp-disunion.h, lisp-union.h: Added warnings on use of VOID_TO_LISP(). lisp.h: Use ERROR_CHECK_STRUCTURES to turn on ERROR_CHECK_TRAPPING_PROBLEMS and ERROR_CHECK_TYPECHECK lisp.h: Add assert_with_message. lisp.h: Add macros for gcproing entire arrays. (You could do this before but it required manual twiddling the gcpro structure.) lisp.h: Add prototypes for new functions defined elsewhere.
author ben
date Tue, 28 May 2002 08:45:36 +0000
parents 943eaba38521
children 804517e16990
line wrap: on
line diff
--- a/src/procimpl.h	Sat May 25 01:55:30 2002 +0000
+++ b/src/procimpl.h	Tue May 28 08:45:36 2002 +0000
@@ -1,5 +1,6 @@
 /* Processes implementation header
    Copyright (C) 1985, 1992, 1993, 1994 Free Software Foundation, Inc.
+   Copyright (C) 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -40,10 +41,11 @@
   void (*finalize_process_data) (Lisp_Process *proc, int for_disksave);
   void (*alloc_process_data) (Lisp_Process *p);
   void (*init_process_io_handles) (Lisp_Process *p,
-				   void* in, void* out, int flags);
+				   void* in, void* out, void *err, int flags);
   int  (*create_process) (Lisp_Process *p,
 			  Lisp_Object *argv, int nargv,
-			  Lisp_Object program, Lisp_Object cur_dir);
+			  Lisp_Object program, Lisp_Object cur_dir,
+			  int separate_err);
   int  (*tooltalk_connection_p) (Lisp_Process *p);
 #ifdef HAVE_SOCKETS
   void (*open_network_stream) (Lisp_Object name, Lisp_Object host,
@@ -65,7 +67,9 @@
   int  (*kill_process_by_pid) (int pid, int sigcode);
   int  (*process_send_eof) (Lisp_Object proc);
   Lisp_Object (*get_tty_name) (Lisp_Process *p);
-  USID (*deactivate_process) (Lisp_Process *p);
+  void (*deactivate_process) (Lisp_Process *p,
+			      USID* in_usid,
+			      USID* err_usid);
   void (*init_process) (void);
 };
 
@@ -98,12 +102,15 @@
   /* (funcall FILTER PROC STRING)  (if FILTER is non-nil)
      to dispose of a bunch of chars from the process all at once */
   Lisp_Object filter;
+  /* (funcall FILTER PROC STRING)  (if FILTER is non-nil)
+     to dispose of a bunch of chars from the stderr of process all at once */
+  Lisp_Object stderr_filter;
   /* (funcall SENTINEL PROCESS) when process state changes */
   Lisp_Object sentinel;
-  /* Buffer that output is going to */
-  Lisp_Object buffer;
+  /* Buffer that output or stderr output is going to */
+  Lisp_Object buffer, stderr_buffer;
   /* Marker set to end of last buffer-inserted output from this process */
-  Lisp_Object mark;
+  Lisp_Object mark, stderr_mark;
   /* Lisp_Int of subprocess' PID, or a cons of
      service/host if this is really a network connection */
   Lisp_Object pid;
@@ -124,19 +131,23 @@
      channel, rather than having a call to make_string.
      This only works if the filter is a subr. */
   char filter_does_read;
-  /* Non-nil means kill silently if Emacs is exited.  */
+  /* Non-zero means kill silently if Emacs is exited.  */
   char kill_without_query;
-  char selected;
+  char in_selected, err_selected;
   /* Event-count of last event in which this process changed status.  */
   volatile int tick;
   /* Event-count of last such event reported.  */
   int update_tick;
+  /* Non-zero if stderr and stdout are separated. */
+  char separate_stderr;
   /* Low level streams used in input and output, connected to child */
   Lisp_Object pipe_instream;
   Lisp_Object pipe_outstream;
+  Lisp_Object pipe_errstream;
   /* Data end streams, decoding and encoding pipe_* streams */
   Lisp_Object coding_instream;
   Lisp_Object coding_outstream;
+  Lisp_Object coding_errstream;
 
   /* Implementation dependent data */
   void *process_data;
@@ -145,6 +156,7 @@
 /* Macros to refer to data connection streams */
 #define DATA_INSTREAM(p) (p)->coding_instream
 #define DATA_OUTSTREAM(p) (p)->coding_outstream
+#define DATA_ERRSTREAM(p) (p)->coding_errstream
 
 /* Random externs from process.c */
 extern Lisp_Object Qrun, Qstop, Qopen, Qclosed;
@@ -168,7 +180,7 @@
 
 Lisp_Object make_process_internal (Lisp_Object name);
 void init_process_io_handles (Lisp_Process *p, void* in,
-			      void* out, int flags);
+			      void* out, void *err, int flags);
 void send_process (Lisp_Object proc,
 		   Lisp_Object relocatable,
 		   const Intbyte *nonrelocatable,